<?php

/**
 * Simple Machines Forum (SMF)
 *
 * @package Naytheet Social Login Code
 * @author Naytheet <naytheet @ simplemachines (dot) org>
 * @copyright 2023 Naytheet
 * @license 3-Clause BSD https://opensource.org/licenses/BSD-3-Clause
 * @version 1.0
 */

if (!defined('SMF'))
	die('Hacking attempt...');

/**
 * naytheet_social_login_load_theme
 *
 * Called by integrate_load_theme hook. Register and/or connect the user via his social login network email, user names and picture.
 * 
 * @return void.
 */
function naytheet_social_login_load_theme() {
	global $boarddir, $modSettings, $user_info;

// user has to be a newcomer
	if (!empty($user_info['id'])) return;

// prepare the random pass
	$pass = random_password(10);

	if (isset($_GET['social_login'])) {

		// prepare the sl name to deal with, along with its keys 
		$network = $_GET['social_login'];
		$id = $modSettings[$network . '_id'];
		$secret = $modSettings[$network . '_secret'];

		// first pass : user authentication
		include($boarddir . '/naytheet_social_login/index.php'); // in case, any SL connection error is kept in $SLErrorMsg.
	} else { // second pass : register the user login data for :

/*		// Facebook
		if (isset($_SESSION["HYBRIDAUTH::STORAGE"]["facebook.authorization_state"])) {
		// don't proceed to the social login if we don't come from the social login page
			if (!isset($_SERVER['HTTP_REFERER'])) return;
			if (($_SERVER['HTTP_REFERER'] != 'https://www.facebook.com/') && ($_SERVER['HTTP_REFERER'] != 'https://m.facebook.com/')) return;

			// prepare long-lived token
			$longLivedToken = isset($modSettings['Facebook_Long_Lived_Token']) ? $modSettings['Facebook_Long_Lived_Token'] : '';

			// get the user's data
	        $json = file_get_contents('https://graph.facebook.com/me?fields=name,picture,email&access_token=' . $longLivedToken);
		    $obj = json_decode($json);
		    $nick = $obj->name;
		    $email = $obj->email;
		    $picture = $obj->picture->data->url;

		    // clean the authenticating side session and register
			unset($_SESSION["HYBRIDAUTH::STORAGE"]);
			registering($nick, $pass, $email, $picture);
		}
*/
		// LinkedIn
		if (isset($_SESSION["HYBRIDAUTH::STORAGE"]["linkedin.authorization_state"])) {
		// don't proceed to the social login if we don't come from the social login page
			if (!isset($_SERVER['HTTP_REFERER'])) return;
			if (($_SERVER['HTTP_REFERER'] != 'https://www.linkedin.com/') && ($_SERVER['HTTP_REFERER'] != 'https://m.linkedin.com/')) return;

			$network = 'LinkedIn';
			$id = $modSettings[$network . '_id'];
			$secret = $modSettings[$network . '_secret'];

			// another turn is needed to get the token
			include($boarddir . '/naytheet_social_login/index.php'); // in case, any SL connection error is kept in $SLErrorMsg.

			// get the user's data
		    $json = file_get_contents('https://api.linkedin.com/v2/me?oauth2_access_token=' . $token["access_token"]);
			$obj = json_decode($json);
			$nick = $obj->localizedFirstName . " " . $obj->localizedLastName;

		    $json = file_get_contents('https://api.linkedin.com/v2/me?projection=(id,profilePicture(displayImage~:playableStreams))&oauth2_access_token=' . $token["access_token"]);
			$obj = json_decode($json);
			$picture = $obj->profilePicture->{"displayImage~"}->elements[0]->identifiers[0]->identifier;

		    $json = file_get_contents('https://api.linkedin.com/v2/clientAwareMemberHandles?q=members&projection=(elements*(handle~))&oauth2_access_token=' . $token["access_token"]);
			$obj = json_decode($json);
			$email = $obj->elements[0]->{"handle~"}->emailAddress;

		    // clean the authenticating side session and register
			unset($_SESSION["HYBRIDAUTH::STORAGE"]);
			registering($nick, $pass, $email, $picture);
		}
	}
}

/**
 * registering
 *
 * Called by naytheet_social_login_load_theme. Registers a new user with a picture url for avatar.
 * @param $nick the user's nickname
 * @param $pass the user's password arbitrarily created in SMF admin page
 * @param $email the user's email
 * @param $picture the picture's url
 * @return void.
 */
function registering($nick, $pass, $email, $picture) {
	global $sourcedir;
	global $user_settings, $smcFunc, $regOptions;

	// prepare the SMF registration with the gotten data
		$regOptions['interface'] = 'guest';
		$regOptions['username'] = $nick;
		$regOptions['email'] = $email;
		$regOptions['password'] = $pass;
		$regOptions['password_check'] = $pass;
		$regOptions['check_reserved_name'] = true;
		$regOptions['check_password_strength'] = false;
		$regOptions['check_email_ban'] = true;
		$regOptions['send_welcome_email'] = false;
		$regOptions['require'] = 'nothing';
		$regOptions['is_activated'] = 1;
		$regOptions['naytheet_social_login']['avatar'] = $picture;

	require_once($sourcedir . '/Subs-Members.php');

	// proceed to registration (won't be registered again if the user already did before) and call naytheet_create_avatar function.
	registerMember($regOptions, true);

	require_once($sourcedir . '/LogInOut.php');

	// procede SMF login with the email provided by the social login page
	$request = $smcFunc['db_query']('', '
		SELECT passwd, password_salt, id_group, additional_groups, id_member, member_name, is_activated, avatar, tfa_backup, tfa_secret, member_name, id_member, passwd_flood
		FROM {db_prefix}members
		WHERE email_address = {string:email}
		LIMIT 1',
		array(
			'email' => $email,
		)
	);
 	$user_settings = $smcFunc['db_fetch_assoc']($request);
	$smcFunc['db_free_result']($request);
	DoLogin();
}

/**
 * naytheet_social_login_general_mod_settings
 *
 * Called by integrate_general_mod_settings hook. Appends a new setting in the admin panel, i.e. SL keys.
 * @param array $config_vars the entire config array at the moment this function was called.
 * @return void.
 */
function naytheet_social_login_general_mod_settings(&$config_vars)
{
	global $txt;

	loadLanguage('NaytheetSocialLogin');

	$config_vars = array_merge($config_vars, array(
		array('title', 'NaytheetSocialLoginTitle')
	));
/*	$config_vars = array_merge($config_vars, array(
		array('text', 'Facebook_id', 'subtext' => $txt['socialLoginFacebookId_desc'], 'help' => 'socialLoginFacebookId_help')
	));
	$config_vars = array_merge($config_vars, array(
		array('text', 'Facebook_secret', 'subtext' => $txt['socialLoginFacebookSecret_desc'], 'help' => 'socialLoginFacebookSecret_help')
	));
	$config_vars = array_merge($config_vars, array(
		array('text', 'Facebook_Long_Lived_Token', 'subtext' => $txt['facebookLongLivedToken_desc'], 'help' => 'facebookLongLivedToken_help')
	));
*/	$config_vars = array_merge($config_vars, array(
		array('text', 'LinkedIn_id', 'subtext' => $txt['socialLoginLinkedInId_desc'], 'help' => 'socialLoginLinkedInId_help')
	));
	$config_vars = array_merge($config_vars, array(
		array('text', 'LinkedIn_secret', 'subtext' => $txt['socialLoginLinkedInSecret_desc'], 'help' => 'socialLoginLinkedInSecret_help')
	));
}

function random_password($length) {
    $alphabet = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890@$#s&*{}\[],=-().+;"\'';
    $pass = array(); //remember to declare $pass as an array
    $alphaLength = strlen($alphabet) - 1; //put the length -1 in cache
    for ($i = 0; $i < $length; $i++) {
        $n = rand(0, $alphaLength);
        $pass[] = $alphabet[$n];
    }
    return implode($pass); //turn the array into a string
}

function naytheet_social_login_helpadmin() {
	loadLanguage('NaytheetSocialLogin');
}

/**
 * naytheet_create_avatar
 *
 * Called by integrate_register hook. Appends the SL avatar in the new user's profile.
 * @param $regOptions the reference provided containing the register vars.
 * @return void.
 */
function naytheet_create_avatar(&$regOptions) {
	if (!empty($regOptions['naytheet_social_login']['avatar']))
			$regOptions['register_vars']['avatar'] = $regOptions['naytheet_social_login']['avatar'];
}